import sys
import random

import ai
from mancala import *


#------------------------------------------------------------------------------

def _PrintAmount(amount):
	if amount != 0:
		print " %2d  " % amount,
	else:
		print "     " ,


def DrawBoard(board):
	print "\n"

	# ruler
	print "      ",
	for binNumber in range(1, BINS_PER_PLAYER+1):
		print "%d    " % binNumber ,
	print "\n"
	print "-"*45
	
	# bins
	print "%2d |" % board.AmountInMancala(ID_PLAYER2),
	for binNumber in range(1, BINS_PER_PLAYER+1):
		_PrintAmount(board.AmountInBin(ID_PLAYER2, binNumber))
	print "\x08|"

	# separator
	print "   |",
	print "="*34,
	print "|"

	# bins
	print "   |",
	for binNumber in range(BINS_PER_PLAYER, 1-1, -1):
		_PrintAmount(board.AmountInBin(ID_PLAYER1, binNumber))
	print "\x08| %2d" % board.AmountInMancala(ID_PLAYER1)

	# ruler
	print "-"*45
	print "\n"
	print " "*6,
	for binNumber in range(BINS_PER_PLAYER, 1-1, -1):
		print "%d    " % binNumber ,
	print "\n"


def GetInput(playerName):
	while True:
		prompt = "%s: press bin number (1..%d), or u to undo, or q to quit, then enter> " % (playerName, BINS_PER_PLAYER)
		inputString = raw_input(prompt)
		if inputString == "q":
			sys.exit()
		if inputString == "u":
			raise "not implemented"

		try:
			binNumber = int(inputString)
			if 1 <= binNumber and binNumber <= BINS_PER_PLAYER:
				return binNumber
		except:
			pass

		print "`%s' was invalid input." % inputString


def CompainAboutInvalidMove():
	print "Invalid move - no pieces in that bin."


def ReportWinner(winnerName):
	if winnerName is None:
		print "Tie!"
	else:
		print "%s wins!" % winnerName


#------------------------------------------------------------------------------

class Human(Player):
	def __init__(self, name = "Player"):
		Player.__init__(self, name)

	def ChooseMove(self, idSide, board):
		return GetInput(self.name)


#------------------------------------------------------------------------------

random.seed(1)

players = (Human(), ai.AiPlayer(100))
idPlayer = ID_PLAYER1
board = Board()

while True:
	DrawBoard(board)

	moves = LegalMoves(idPlayer, board)
	if len(moves) == 0:
		idWinner = AssignRemainingPiecesAndDetermineWinner(board)
		DrawBoard(board)
		
		winnerName = None
		if idWinner is not None:
			winnerName = players[idWinner].Name()
		ReportWinner(winnerName)
		break

	player = players[idPlayer]
	binNumber = player.ChooseMove(idPlayer, board)

	if binNumber not in moves:
		CompainAboutInvalidMove()
		continue

	idPlayer = CarryOutMove(idPlayer, binNumber, board)
