from mancala import *

# all scores are from the perspective of the searching player - the maximizer.
# due to negamax formulation, scoring must be symmetric.

# exactly one of these is added for each bin:
SCORE_PIECE          = 10		# per piece
SCORE_SMALL_BIN      = 120
SCORE_END_IN_MANCALA = 300
SCORE_ENDZONE_CLEAR  = 300

SCORE_MANCALA        = 1000		# per piece
SCORE_WON            = 100000	# > sum of any other score


def _EvaluateBoardForSide(idSide, board):
	score = 0

	score += SCORE_MANCALA * board.AmountInMancala(idSide)

	for binNumber in range(1, BINS_PER_PLAYER+1):
		amount = board.AmountInBin(idSide, binNumber)
		if amount < binNumber:
			score += SCORE_SMALL_BIN
		elif amount == binNumber:
			score += SCORE_END_IN_MANCALA
		else:
			score += SCORE_PIECE * amount

	if board.AmountInBin(idSide, 1) <= 1:
		score += SCORE_ENDZONE_CLEAR

	return score


def EvaluateBoard(idSide, board):
	score = _EvaluateBoardForSide(idSide, board) - _EvaluateBoardForSide(IdOpponent(idSide), board)
	return score



def _EvaluateGameOverForSide(idSide, board):
	amount = board.AmountInMancala(idSide)
	for binNumber in range(1, BINS_PER_PLAYER+1):
		amount += board.AmountInBin(idSide, binNumber)
	return amount


def EvaluateGameOver(idSide, board):
	score = _EvaluateGameOverForSide(idSide, board) - _EvaluateGameOverForSide(IdOpponent(idSide), board)
	if score > 0:
		score += SCORE_WON
	else:
		score -= SCORE_WON
	return score