import sys
import operator
import random

import eval
import copy
import mancala


nodes = None

# note: we currently don't bother with principal variation search or
# ordering improvements such as the killer heuristic because the fanout
# factor in Mancala is only 6.

def Search(maxPly, percentTolerance, idPlayer, board):
	global nodes

	alpha = -sys.maxint-1
	beta  = +sys.maxint
	nodes = 0
	
	moves = []
	for binNumber in mancala.LegalMoves(idPlayer, board):
		board2 = copy.deepcopy(board)
		idSide2 = mancala.CarryOutMove(idPlayer, binNumber, board2)
		score = NegamaxR(maxPly, alpha, beta, idPlayer, idSide2, board2)
		moves.append((binNumber, score))

	moves.sort(key = operator.itemgetter(1), reverse = True)
	print "(nodes=%d) %d:" % (nodes, idPlayer),
	print moves

	bestScore = moves[0][1]
	tolerance = abs(bestScore * percentTolerance / 100)
	moves = [move for move in moves if abs(move[1]-bestScore) < tolerance]
	print moves
	
	return random.choice(moves)[0]


def NegamaxR(maxPly, alpha, beta, idPlayer, idSide, board):
	global nodes
	if maxPly == 0:
		nodes += 1
		return eval.EvaluateBoard(idPlayer, board)

	moves = mancala.LegalMoves(idSide, board)
	if len(moves) == 0:
		nodes += 1
		return eval.EvaluateGameOver(idPlayer, board)

	scores = []
	for binNumber in moves:
		board2 = copy.deepcopy(board)
		idSide2 = mancala.CarryOutMove(idSide, binNumber, board2)
		score = NegamaxR(maxPly-1, alpha, beta, idPlayer, idSide2, board2)
		scores.append(score)

	if idSide == idPlayer:
		return max(scores)
	else:
		return min(scores)
